from pathlib import Path
import os
import shutil
from dataclasses import dataclass

from markdown_it import MarkdownIt


@dataclass(frozen=True)
class Config:
    source_dir: Path
    build_dir: Path


def _read_if_exists(path: Path) -> str:
    if path.exists():
        with open(path, "r", encoding="utf-8") as f:
            return f.read()
    return ""


def _replace_content(src: Path, dst: Path):
    if src.exists():
        if dst.exists():
            shutil.rmtree(dst)
        shutil.copytree(src, dst)


def _write_file(path: Path, content: str):
    os.makedirs(path.parent, exist_ok=True)
    with open(path, "w", encoding="utf-8") as f:
        f.write(content)


def _read_file(path: Path) -> str:
    with open(path, "r", encoding="utf-8") as f:
        return f.read()


def generate(config: Config):

    static_dir = config.source_dir / "static"
    build_static_dir = config.build_dir / "static"
    _replace_content(static_dir, build_static_dir)

    template_dir = config.source_dir / "templates"
    header = _read_if_exists(template_dir / "header.html")
    footer = _read_if_exists(template_dir / "footer.html")

    parser = MarkdownIt()
    sources = config.source_dir.glob("**/*.md")

    for source in sources:
        source_dir = source.parent.relative_to(config.source_dir)
        output_path = config.build_dir / source_dir / (str(source.stem) + ".html")

        content = _read_file(source)
        output = parser.render(content)
        _write_file(output_path, header + output + footer)
